/*
   Copyright (C) 2006 by krasnop@bellsouth.net (Alexei Krasnopolski)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License. 
*/

#include "ISAObject.hpp"

ISAObject::ISAObject( DWORD prio, DWORD intNumber ) : AObject( prio )
{
  registerInterrupt( intNumber );
}

ISAObject::ISAObject( DWORD prio, DWORD intNumber, Hsm* hsm ) : AObject( prio, hsm )
{
  registerInterrupt( intNumber );
}

void
ISAObject::registerInterrupt( DWORD intNumber )
{
  interruptTable[intNumber] = this;
}

AO_STACK *
processInterrupt( DWORD iN, AO_STACK * stp )
{
  static DWORD nestedInterrupt = 0;
  nestedInterrupt++;                    // count nested entering to the interrupt processing

  ISAObject * obj;
  if( iN != SCHEDULER_INTERRUPT_NUMBER )  
  {                                     // this is not scheduler. 
    EXIT_CRITICAL()                     // unmask interrupts

    obj = ISAObject::interruptTable[iN]; // get IS Active Object corresponded iN
    if( obj != 0 )
    {
      obj->serviceInterrupt( stp );      // run interrupt processing
    }
    
    ENTER_CRITICAL()                     // mask interrupts
  }

  if( --nestedInterrupt == 0 )           // need epilog code ?
  {
    obj = ISAObject::interruptTable[SCHEDULER_INTERRUPT_NUMBER];  // if it is not nested interrupt
    return obj->serviceInterrupt( stp );                          // then run scheduler 
  }
  return stp;                                                     // cause context switch
}
