/*
   Copyright (C) 2006 by krasnop@bellsouth.net (Alexei Krasnopolski)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License. 
*/

#ifndef _AOBJECT_H
#define _AOBJECT_H

#include "commonDef.hpp"
#include "ListenerList.hpp"
#include "RingBuffer.hpp"
#include "Thread.hpp"
#include "Hsm.h"

/**
 * Class is responsible for main functionality of the active object such as
 * sending and receiving of events (or messages), managing of the event buffer
 * and state machine object. This is a superclass for all RTOS components.
 */
class AObject : public Thread
{
/***************** Fields ***************/
 private:
/** Reference to state machine object associated with this active object. */
   Hsm * stateMachine;
/** List of the active objects that are registered to receive events from this active object.*/
   ListenerList list;
/** ringBuffer keeps the incoming events for father processing.*/
   RingBuffer   ringBuffer;

/***************** Methods ***************/
 private:
/** Method run executes infinite loop, retrives events from buffer and passes its to
 *  state machine
 */ 
   void run();

 protected:
/** Method that processes the coming event. As the method is virtual 
 *  the subclasses can override it. The method for this class just passes
 *  event to state machine.
 */
   virtual DWORD processEvent( Event * );

 public:
/** Constructors
 *  @param prio - priority of the active object.
 *  @param hsm - reference to associated hierarchical state machine.
 */
        AObject( DWORD prio );
        AObject( DWORD prio, Hsm * hsm );
/** Method publiches event generated by the active object to all its listeners 
 *  @param e - event to publish.
 */
   void publishEvent( Event * e );
/** adds an active object to listener list of the given active object.
 *  @param ao - reference to active object.
 */
   void addListener( AObject * ao );
/** removes an active object from listener list of the given active object.
 *  @param ao - reference to active object.
 */
   void removeListener( AObject * ao );
/** When other active object wants to send an event to this active object,
 *  the first have to invoke this method of given active object.
 *  @param e - incoming event.
 */
   void receiveEvent( Event * e );
/** gets reference to associted state machine.
 *  @return Hsm * - reference to state machine instance. 
 */   
   inline Hsm * getStateMachine(){ return stateMachine; };
/** Function for debuging. It returns level of loading of ring buffer. 
 *  @return int - number of elements available for reading
 */
   inline int  bufferLoad(){ return ringBuffer.bufferLoad(); };
};

#endif
