/*
   Copyright (C) 2006 by krasnop@bellsouth.net (Alexei Krasnopolski)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License. 
*/

#ifndef _RINGBUFFER_H
#define _RINGBUFFER_H

#include "commonDef.hpp"

class AObject;

/**
 * Union Control is simple data structure for keeping either reference to Active Object
 * or integer number
 */
union Control
{
  Control() : data( 0 ) {}
  Control(  DWORD d ) : data( d ) {}
  Control( AObject * ao ) : pAObject( ao ) {}

  AObject * pAObject;
  DWORD     data;
};

/**
 *  structure Event incapsulates field of Control type and field EventID type
 */ 
struct Event
{
  Event() : eventId( no ) {}
  Event( AObject * ao, EventID eid ) : ctrl( ao ), eventId( eid ) {}
  Event( DWORD data, EventID eid ) : ctrl( data ), eventId( eid ) {}
  Event( const Event &ev ){ ctrl.data = ev.ctrl.data; eventId = ev.eventId; }
  void operator = ( const Event &ev ){ ctrl.data = ev.ctrl.data; eventId = ev.eventId; }

  Control ctrl;
  EventID eventId;
};

/**
 * Class RingBuffer keeps incoming events. Pointer wrPo points to first empty element of buffer,
 * ready to accept an incoming event. rdPo points to first buffer element ready to be read out.
 * Pointers are moving throu buffer cyclically when pointer reach the buttom of buffer it go to
 * top.
 */
class RingBuffer
{
/***************** Fields ***************/
 private:
/** wrPo keeps index of element of queue that empty and ready to accept new event.*/
  int wrPo;
/** rdPo keeps index of element of queue that ready to be read.*/
  int rdPo;
/** Size of queue.*/
  int N;
/** Array of Event*/
  Event queue[AO_RINGBUFFER_LENGTH];

/***************** Methods ***************/

 public:
/** Constructor that creates RingBuffer with size of queue.
 *  @param int size - length of queue.
 */
      RingBuffer( int size );
/** Default constructor.*/
      RingBuffer();
/** The method writes event ev to buffer.
 *  @param Event * ev - reference to incoming event
 *  @return int - 1 if success, 0 if buffer is full. 
 */
  int write( Event * ev );
/** The method reads first available element of buffer and save it in (*ev).
 *  @param Event * ev - reference to destination Event object
 *  @return int - 1 if success, 0 if buffer is empty.  
 */
  int read( Event * ev );
/** The method returns amount of elements that are available for reading.
 *  ( for debugging use )
 */
  int bufferLoad();
};

#endif
