/*
   Copyright (C) 2006 by krasnop@bellsouth.net (Alexei Krasnopolski)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License. 
*/

#include "RingBuffer.hpp"

RingBuffer::RingBuffer( int n )
{
  N = ( n > AO_RINGBUFFER_LENGTH ) ? AO_RINGBUFFER_LENGTH : n;
  wrPo = 0;
  rdPo = N - 1;
}

RingBuffer::RingBuffer()
{
  N = AO_RINGBUFFER_LENGTH;
  wrPo = 0;
  rdPo = N - 1;
}

int
RingBuffer::write( Event * event )
{
  ENTER_CRITICAL()          // mask interrupt and avoid double enter to this point
  if( wrPo != rdPo )        // is buffer full ?
  {
    queue[wrPo] = *event;
    (++wrPo) %= N;          // set pointer to next element and revert to 0 if wrPo >= N (implements a ring) 
    EXIT_CRITICAL()
    return 1;
  }
  EXIT_CRITICAL()
  return 0;
}

int
RingBuffer::read( Event * event )
{
  ENTER_CRITICAL()               
  if( wrPo != (rdPo + 1)%N ) // is a buffer empty ?
  {
    (++rdPo) %= N;          // go to next element and revert to 0 if rdPo >= N (implements a ring)
    *event = queue[rdPo];
    EXIT_CRITICAL()
    return 1;
  }
  EXIT_CRITICAL()                
  return 0;
}

int
RingBuffer::bufferLoad()
{
  ENTER_CRITICAL()
  int i = wrPo - rdPo;
  EXIT_CRITICAL()
  return (i - 1 + N) % N;
}
